@@ -39,14 +39,15 @@ control over the behaviour of neurons.
 """
 
 __all__ = ['MultipleSpikeGeneratorGroup', 'SpikeGeneratorGroup', 'PulsePacket',
-           'PoissonGroup', 'OfflinePoissonGroup', 'PoissonInputs']
+           'PoissonGroup', 'OfflinePoissonGroup', 'PoissonInput']
 
 from neurongroup import *
 from threshold import *
 from stateupdater import *
 from units import *
 import random as pyrandom
-from numpy import where, array, zeros, ones, inf, nonzero, tile, sum, isscalar, cumsum
+from numpy import where, array, zeros, ones, inf, nonzero, tile, sum, isscalar,\
+                  cumsum, hstack, bincount,  ceil, ndarray, ascontiguousarray
 from copy import copy
 from clock import guess_clock
 from utils.approximatecomparisons import *
@@ -182,14 +183,13 @@ class SpikeGeneratorGroup(NeuronGroup):
         An object specifying which neurons should fire and when. It can be a container
         such as a ``list``, containing tuples ``(i,t)`` meaning neuron ``i`` fires at
         time ``t``, or a callable object which returns such a container (which
-        allows you to use generator objects, see below). ``i`` can be an integer
+        allows you to use generator objects even though this is slower, see below). ``i`` can be an integer
         or an array (list of neurons that spike at the same time).
         If ``spiketimes`` is not a list or tuple, the pairs ``(i,t)`` need to be
         sorted in time. You can also pass a numpy array
         ``spiketimes`` where the first column of the array
         is the neuron indices, and the second column is the times in
-        seconds. WARNING: units are not checked in this case, and you need to
-        ensure that the spikes are sorted.
+        seconds. Alternatively you can pass a tuple with two arrays, the first one being the neuron indices and the second one times. WARNING: units are not checked in this case, the time array should be in seconds.
     ``clock``
         An optional clock to update with (omit to use the default clock).
     ``period``
@@ -198,7 +198,7 @@ class SpikeGeneratorGroup(NeuronGroup):
         with a period as they cannot be reinitialised.
     ``gather=False``
         Set to True if you want to gather spike events that fall in the same
-        timestep (makes the simulation faster if you have many events).
+        timestep. (Deprecated since Brian 1.3.1)
     ``sort=True``
         Set to False if your spike events are already sorted.
     
@@ -226,7 +226,7 @@ class SpikeGeneratorGroup(NeuronGroup):
         P = SpikeGeneratorGroup(10,nextspike())
     
     This would give a neuron group ``P`` with 10 neurons, where a random one
-    of the neurons fires at an average rate of one every 5ms.
+    of the neurons fires at an average rate of one every 5ms. Please note that as of 1.3.1, this behavior is preserved but will run slower than initializing with arrays, or lists.
     
     **Notes**
     
@@ -234,6 +234,8 @@ class SpikeGeneratorGroup(NeuronGroup):
     spikes will be discarded. If you want them to stack, consider using the less efficient
     :class:`MultipleSpikeGeneratorGroup` object instead. A warning will be issued if this
     is detected.
+
+    Also, if you want to use a SpikeGeneratorGroup with many spikes and/or neurons, please use an initialization with arrays.
     
     Also note that if you pass a generator, then reinitialising the group will not have the
     expected effect because a generator object cannot be reinitialised. Instead, you should
@@ -245,41 +247,154 @@ class SpikeGeneratorGroup(NeuronGroup):
     Whenever P is reinitialised, it will call ``nextspike()`` to create the required spike
     container.
     """
-    def __init__(self, N, spiketimes, clock=None, period=None, gather=False, sort=True):
+    def __init__(self, N, spiketimes, clock=None, period=None, 
+                 sort=True, gather=None):
         clock = guess_clock(clock)
-        if gather: # assumes spike times are sorted
-            spiketimes=self.gather(spiketimes,clock.dt)
-            sort=False
-        thresh = SpikeGeneratorThreshold(N, spiketimes, period=period, sort=sort)
+        self.N = N
         self.period = period
-        NeuronGroup.__init__(self, N, model=LazyStateUpdater(), threshold=thresh, clock=clock)
-
-    def gather(self,spiketimes,dt):
-        # Gathers spike events in the same timestep
-        # Assumes spikes are sorted
-        if isinstance(spiketimes, (list, tuple)):
-            spiketimes=array(spiketimes)
-        times=array(spiketimes[:,1]/dt,dtype=int) # in units of dt
-        neurons=array(spiketimes[:,0],dtype=int)
-        u,indices=numpy.unique(times,return_index=True) # split over timesteps
-        new_spiketimes=[]
-        for i in range(len(u)-1):
-            new_spiketimes.append((neurons[indices[i]:indices[i+1]],float(u[i])*dt))
-        new_spiketimes.append((neurons[indices[-1]:],float(u[-1])*dt))
-        return new_spiketimes
+        if gather:
+            log_warn('brian.SpikeGeneratorGroup', 'SpikeGeneratorGroup\'s gather keyword use is deprecated')
+        fallback = False # fall back on old SpikeGeneratorThreshold or not
+        if isinstance(spiketimes, list):
+            # spiketimes is a list of (i,t)
+            if len(spiketimes):
+                idx, times = zip(*spiketimes)
+            else:
+                idx, times = [], []
+            # the following try ... handles the case where spiketimes has index arrays
+            # e.g spiketimes = [([0, 1], 0 * msecond), ([0, 1, 2], 2 * msecond)]
+            # Notes:
+            # - if there is always the same number of indices by array, its simple, it's just a matter of flattening
+            # - if not, then it requires a for loop, and it's done in the except
+            try:
+                idx = array(idx, dtype = float)
+                times = array(times, dtype = float)
+                if idx.ndim > 1:
+                    # simple case
+                    times = tile(times.reshape((len(times), 1)), (idx.shape[1], 1)).flatten()
+                    idx = idx.flatten()
+            except ValueError:
+                new_idx = []
+                new_times = []
+                for k, item in enumerate(idx):
+                    if isinstance(item, list):
+                        new_idx += item # append indices
+                        new_times += [times[k]]*len(item)
+                    else:
+                        new_times += [times[k]]
+                        new_idx += [item]
+                idx = array(new_idx, dtype  = float)
+                times = new_times
+                times = array(times, dtype = float)
+        elif isinstance(spiketimes, tuple):
+            # spike times is a tuple with idx, times in arrays
+            idx = spiketimes[0]
+            times = spiketimes[1]
+        elif isinstance(spiketimes, ndarray):
+            # spiketimes is a ndarray, with first col is index and second time
+            idx = spiketimes[:,0]
+            times = spiketimes[:,1]
+        else:
+            log_warn('brian.SpikeGeneratorGroup', 'Using (slow) threshold because spiketimes is assumed to be a generator/iterator')
+            # spiketimes is a callable object, so falling back on old SpikeGeneratorThreshold
+            fallback = True
 
+        if not fallback:
+            thresh = FastSpikeGeneratorThreshold(N, idx, times, dt=clock.dt, period=period)
+        else:
+            thresh = SpikeGeneratorThreshold(N, spiketimes, period=period, sort=sort)
+        
+        if not hasattr(self, '_initialized'):
+            NeuronGroup.__init__(self, N, model=LazyStateUpdater(), threshold=thresh, clock=clock)
+            self._initialized = True
+        else:
+            self._threshold = thresh
+ 
     def reinit(self):
         super(SpikeGeneratorGroup, self).reinit()
         self._threshold.reinit()
 
-    spiketimes = property(fget=lambda self:self._threshold.spiketimes,
-                          fset=lambda self, value: self._threshold.set_spike_times(self._threshold.N, value, self._threshold.period))
-
-    def __repr__(self):
-        return "SpikeGeneratorGroup"
+    @property
+    def spiketimes(self):
+        return self._threshold.spiketimes
+    
+    @spiketimes.setter
+    def spiketimes(self, values):
+        self.__init__(self.N, values, period = self.period)
 
+class FastSpikeGeneratorThreshold(Threshold):
+    '''
+    A faster version of the SpikeGeneratorThreshold where spikes are processed prior to the run (offline). It replaces the SpikeGeneratorThreshold as of 1.3.1.
+    '''
+    ## Notes:
+    #  - N is ignored (should it not?)
+    def __init__(self, N, addr, timestamps, dt = None, period=None):
+        self.set_offsets(addr, timestamps, dt = dt)
+        self.period = period
+        self.dt = dt
+        self.reinit()
+        
+    def set_offsets(self, I, T, dt = 1000):
+        # Convert times into integers
+        T = array(ceil(T/dt), dtype=int)
+        # Put them into order
+        # We use a field array to sort first by time and then by neuron index
+        spikes = zeros(len(I), dtype=[('t', int), ('i', int)])
+        spikes['t'] = T
+        spikes['i'] = I
+        spikes.sort(order=('t', 'i'))
+        T = ascontiguousarray(spikes['t'])
+        self.I = ascontiguousarray(spikes['i'])
+        # Now for each timestep, we find the corresponding segment of I with
+        # the spike indices for that timestep.
+        # The idea of offsets is that the segment offsets[t]:offsets[t+1]
+        # should give the spikes with time t, i.e. T[offsets[t]:offsets[t+1]]
+        # should all be equal to t, and so then later we can return
+        # I[offsets[t]:offsets[t+1]] at time t. It might take a bit of thinking
+        # to see why this works. Since T is sorted, and bincount[i] returns the
+        # number of elements of T equal to i, then j=cumsum(bincount(T))[t]
+        # gives the first index in T where T[j]=t.
+        if len(T):
+            self.offsets = hstack((0, cumsum(bincount(T))))
+        else:
+            self.offsets = array([])
+    
+    def __call__(self, P):
+        t = P.clock.t
+        if self.period is not None:
+            cp = int(t / self.period)
+            if cp > self.curperiod:
+                self.reinit()
+                self.curperiod = cp
+            t = t - cp * self.period
+        dt = P.clock.dt
+        t = int(round(t/dt))
+        if t+1>=len(self.offsets):
+            return array([], dtype=int)
+        return self.I[self.offsets[t]:self.offsets[t+1]]
+    
+    def reinit(self):
+        self.curperiod = -1
+        
+    @property
+    def spiketimes(self):
+        # this is a pain to do! retrieve spike times from offsets
+        res = []
+        for k in range(len(self.offsets)-1):
+            idx = self.I[self.offsets[k]:self.offsets[k+1]]
+            ts = [k*self.dt]*len(idx)
+            res += zip(idx, ts)
+        return res
 
 class SpikeGeneratorThreshold(Threshold):
+    """
+    Old threshold object for the SpikeGeneratorGroup
+    
+    **Notes**
+
+    This version of the SpikeGeneratorThreshold object is deprecated, since version 1.3.1 of Brian it has been replaced in most cases by the FastSpikeGeneratorThreshold. 
+    This is kept only as a fallback object for when a SpikeGeneratorGroup object is initialized with a generator or an iterator object (see the doc for SpikeGeneratorGroup for more details). Please note that since this implementation is slower, using a static data structure as an input to a SpikeGeneratorGroup is advised.
+    """
     def __init__(self, N, spiketimes, period=None, sort=True):
         self.set_spike_times(N, spiketimes, period=period, sort=sort)
 
@@ -441,7 +556,7 @@ class OfflinePoissonGroup(object): # This is weird, there is only an init method
         self.spiketimes = zip(neurons, spikes)
 
 
-# Used in PoissonInputs below
+# Used in PoissonInput below
 class EmptyGroup(object):
     def __init__(self, clock):
         self.clock = clock
@@ -449,99 +564,149 @@ class EmptyGroup(object):
         return None
 
 
-class PoissonInputs(Connection):
+class PoissonInput(Connection):
+    """
+    Adds a Poisson input to a NeuronGroup. Allows to efficiently simulate a large number of 
+    independent Poisson inputs to a NeuronGroup variable, without simulating every synapse
+    individually. The synaptic events are generated randomly during the simulation and
+    are not preloaded and stored in memory (unless record=True is used).
+    All the inputs must target the same variable, have the same frequency and same synaptic
+    weight. You can use as many PoissonInput objects as you want, even targetting a same NeuronGroup.
+    There is the possibility to consider time jitter in the presynaptic spikes, and
+    synaptic unreliability. The inputs can also be recorded if needed. Finally, all
+    neurons from the NeuronGroup receive independent realizations of Poisson spike trains,
+    except if the keyword freeze=True is used, in which case all neurons receive the same
+    Poisson input.
+    
+    **Initialised as:** ::
+    
+        PoissonInput(target[, N][, rate][, weight][, state][, jitter][, reliability][, copies][, record][, freeze])
+    
+    with arguments:
+    
+    ``target``
+        The target :class:`NeuronGroup`
+    ``N``
+        The number of independent Poisson inputs
+    ``rate``
+        The rate of each Poisson process
+    ``weight``
+        The synaptic weight
+    ``state``
+        The name or the index of the synaptic variable of the :class:`NeuronGroup`      
+    ``jitter``
+        is ``None`` by default. There is the possibility to consider ``copies`` presynaptic
+        spikes at each Poisson event, randomly shifted according to an exponential law
+        with parameter ``jitter=taujitter`` (in second).
+    ``reliability`` 
+        is ``None`` by default. There is the possibility to consider ``copies`` presynaptic
+        spikes at each Poisson event, where each of these spikes is unreliable, i.e. it occurs
+        with probability ``jitter=alpha`` (between 0 and 1).
+    ``copies``
+        The number of copies of each Poisson event. This is identical to ``weight=copies*w``, except
+        if ``jitter`` or ``reliability`` are specified.
+    ``record``
+        ``True`` if the input has to be recorded. In this case, the recorded events are
+        stored in the ``recorded_events`` attribute, as a list of pairs ``(i,t)`` where ``i`` is the
+        neuron index and ``t`` is the event time.
+    ``freeze``
+        ``True`` if the input must be the same for all neurons of the :class:`NeuronGroup`
+    """    
     _record = []
     
-    def __init__(self, target, sameinputs=[], *inputs, **kwds):
-        """
-        Adds Poisson inputs to a NeuronGroup.
-        
-        Initialised with arguments:
-        
-        ``target``
-            The target NeuronGroup
-        
-        ``sameinputs = []``
-            The list of the inputs indices which are assumed to be identical for all neurons.
-        
-        ``inputs``
-            The list of the Poisson inputs, as a list of tuples (n, f, w, state)
-            where n is the number of Poisson spike trains, f their rate, w the
-            synaptic weight, and state the name of the state to connect the input to.
-        """
+    def __init__(self, target, N=None, rate=None, weight=None, state=None,
+                  jitter=None, reliability=None, copies=1,
+                  record=False, freeze=False):
         self.source = EmptyGroup(target.clock)
         self.target = target
         self.N = len(self.target)
         self.clock = target.clock
-        self.inputs = inputs
         self.delay = None
         self.iscompressed = True
-        self.W = zeros((len(inputs), self.N))
-        self.sameinputs = sameinputs
-        if 'record' in kwds.keys():
-            self._record = kwds['record']
-            if type(self._record) is int:
-                self._record = [self._record]
-
-        self.stateindex = dict()
         self.delays = None # delay to wait for the j-th synchronous spike to occur after the last sync event, for target neuron i
         self.lastevent = -inf * ones(self.N) # time of the last event for target neuron i
-        for i in xrange(len(self.inputs)):
-            state = self.inputs[i][3]
-            if isinstance(state, str): # named state variable
-                self.stateindex[state] = target.get_var_index(state)
-            else:
-                self.stateindex[state] = state
-            w = self.inputs[i][2]
-            if type(w) is tuple:
-                if w[0] == 'jitter':
-                    self.delays = zeros((w[2], self.N))
         self.events = []
         self.recorded_events = []
 
+        self.n = N
+        self.rate = rate
+        self.w = weight
+        self.var = state
+        self._jitter = jitter
+        
+        if jitter is not None:
+            self.delays = zeros((copies, self.N))
+        
+        self.reliability = reliability
+        self.copies = copies
+        self.record = record
+        self.frozen = freeze
+        
+        if (jitter is not None) and (reliability is not None):
+            raise Exception("Specifying both jitter and reliability is currently not supported.")
+
+        if isinstance(state, str): # named state variable
+            self.index = self.target.get_var_index(state)
+        else:
+            self.index = state
+
+    @property
+    def jitter(self):
+        return self._jitter
+
+    @jitter.setter
+    def jitter(self, value):
+        self._jitter = value
+        if value is not None:
+            self.delays = zeros((self.copies, self.N))
+
     def propagate(self, spikes):
-        current = zeros(self.N)
         i = 0
-        for (n, f, w, state) in self.inputs:
-            state = self.stateindex[state]
-            if type(w) is not tuple:
-                if i in self.sameinputs:
-                    rnd = binomial(n=n, p=f * self.clock.dt)
-                    self.target._S[state, :] += w * rnd
-                    if rnd > 0:
-                        self.events.append(self.clock.t)
-                else:
-                    rnd = binomial(n=n, p=f * self.clock.dt, size=(self.N))
-                    self.target._S[state, :] += w * rnd
-                    ind = nonzero(rnd>0)[0]
-                    if i in self._record and len(ind)>0:
-                        self.recorded_events.append((ind[0], self.clock.t))
+        
+        n = self.n
+        f = self.rate
+        w = self.w
+        var = self.var
+        jitter = self.jitter
+        reliability = self.reliability
+        record = self.record
+        frozen = self.frozen
+        state = self.index
+        
+        if (jitter==None) and (reliability==None):
+            if frozen:
+                rnd = binomial(n=n, p=f * self.clock.dt)
+                self.target._S[state, :] += w * rnd
+                if rnd > 0:
+                    self.events.append(self.clock.t)
             else:
-                # if w is a tuple, it is ('synapse', w, pmax, alpha) and there are
-                # binomial(pmax, alpha) synchronous spikes then
-                # or it is ('jitter', w, pmax, jitter) and the synchronous
-                # spikes are shifted by an exponential value with parameter jitter
-                if w[0] == 'synapse':
-                    if (w[2] > 0) & (w[3] > 0):
-                        weff = w[1] * binomial(n=w[2], p=w[3])
-                        self.target._S[state, :] += weff * binomial(n=n, p=f * self.clock.dt, size=(self.N))
-                elif w[0] == 'jitter':
-                    p = w[2]
-                    if (p > 0) & (f > 0):
-                        jitter = w[3]
-                        k = binomial(n=n, p=f * self.clock.dt, size=(self.N)) # number of synchronous events here, for every target neuron
-                        syncneurons = (k > 0) # neurons with a syncronous event here
-                        self.lastevent[syncneurons] = self.clock.t
-                        if jitter == 0.0:
-                            self.delays[:, syncneurons] = zeros((p, sum(syncneurons)))
-                        else:
-                            self.delays[:, syncneurons] = exponential(scale=jitter, size=(p, sum(syncneurons)))
-                        # Delayed spikes occur now
-                        lastevent = tile(self.lastevent, (p, 1))
-                        b = (abs(self.clock.t - (lastevent + self.delays)) <= (self.clock.dt / 2) * ones((p, self.N))) # delayed spikes occurring now
-                        weff = sum(b, axis=0) * w[1]
-                        self.target._S[state, :] += weff
-            i += 1
+                rnd = binomial(n=n, p=f * self.clock.dt, size=(self.N))
+                self.target._S[state, :] += w * rnd
+                ind = nonzero(rnd>0)[0]
+                if record and len(ind)>0:
+                    self.recorded_events.append((ind[0], self.clock.t))
+        elif (jitter is not None):
+            p = self.copies
+            taujitter = jitter
+            if (p > 0) & (f > 0):
+                k = binomial(n=n, p=f * self.clock.dt, size=(self.N)) # number of synchronous events here, for every target neuron
+                syncneurons = (k > 0) # neurons with a syncronous event here
+                self.lastevent[syncneurons] = self.clock.t
+                if taujitter == 0.0:
+                    self.delays[:, syncneurons] = zeros((p, sum(syncneurons)))
+                else:
+                    self.delays[:, syncneurons] = exponential(scale=taujitter, size=(p, sum(syncneurons)))
+                # Delayed spikes occur now
+                lastevent = tile(self.lastevent, (p, 1))
+                b = (abs(self.clock.t - (lastevent + self.delays)) <= (self.clock.dt / 2) * ones((p, self.N))) # delayed spikes occurring now
+                weff = sum(b, axis=0) * w
+                self.target._S[state, :] += weff
+        elif (reliability is not None):
+            p = self.copies
+            alpha = reliability
+            if (p > 0) & (alpha > 0):
+                weff = w * binomial(n=p, p=alpha)
+                self.target._S[state, :] += weff * binomial(n=n, p=f * self.clock.dt, size=(self.N))
 
 def _test():
     import doctest
