@@ -17,15 +17,17 @@ except (ImportError, ValueError):
 from bufferable import Bufferable
 from prefs import get_samplerate
 from db import dB, dB_type, dB_error, gain
+from scipy.signal import fftconvolve, lfilter
+from scipy.misc import factorial
 
 __all__ = ['BaseSound', 'Sound',
            'pinknoise','brownnoise','powerlawnoise',
-           'whitenoise', 'tone', 'click', 'clicks', 'silence', 'sequence', 'harmoniccomplex',
-           'loadsound', 'savesound', 'play',
+           'whitenoise', 'irns', 'irno', 
+           'tone', 'click', 'clicks', 'silence', 'sequence', 'harmoniccomplex',
+           'loadsound', 'savesound', 'play', 'vowel'
            ]
 
 
-
 _mixer_status = [-1,-1]
 
 class BaseSound(Bufferable):
@@ -87,6 +89,7 @@ class Sound(BaseSound, numpy.ndarray):
     .. automethod:: click
     .. automethod:: clicks
     .. automethod:: harmoniccomplex
+    .. automethod:: vowel
     
     **Timing and sequencing**
     
@@ -119,6 +122,8 @@ class Sound(BaseSound, numpy.ndarray):
     
     .. autoattribute:: level
     .. automethod:: atlevel
+    .. autoattribute:: maxlevel
+    .. automethod:: atmaxlevel
     
     **Ramping**
     
@@ -155,7 +160,9 @@ class Sound(BaseSound, numpy.ndarray):
         elif isinstance(data, str):
             if duration is not None:
                 raise ValueError('Cannot specify duration when initialising Sound from file.')
-            x = Sound.load(data, samplerate=samplerate)
+            if samplerate is not None:
+                raise ValueError('Cannot specify samplerate when initialising Sound from a file.')
+            x = Sound.load(data)
             samplerate = x.samplerate
         elif callable(data):
             samplerate = get_samplerate(samplerate)
@@ -243,33 +250,43 @@ class Sound(BaseSound, numpy.ndarray):
         return self.__setitem__(slice(start, stop), seq)
     
     def __getitem__(self,key):
-        channel=slice(None)
-        if isinstance(key,tuple):
-            channel=key[1]
-            key=key[0]
+        channel = slice(None)
+        if isinstance(key, tuple):
+            channel = key[1]
+            key = key[0]
 
-        if isinstance(key,int):
-            return np.ndarray.__getitem__(self,key)
-        if isinstance(key,Quantity):
-            return np.ndarray.__getitem__(self,round(key*self.samplerate))
+        if isinstance(key, int):
+            return np.ndarray.__getitem__(self, key)
+        if isinstance(key, float):
+            return np.ndarray.__getitem__(self, round(key*self.samplerate))
 
         sliceattr = [v for v in [key.start, key.stop] if v is not None]
-        slicedims = array([units.have_same_dimensions(flag,second) for flag in sliceattr])
+        slicedims = array([units.have_same_dimensions(flag, second) for flag in sliceattr])
 
-        if not slicedims.any():
+        attrisint = array([isinstance(v, int) for v in sliceattr])
+        s = sum(attrisint)
+        if s!=0 and s!=len(sliceattr):
+            raise ValueError('Slice attributes must be all ints or all times')
+        if s==len(sliceattr): # all ints
             start = key.start or 0
             stop = key.stop or self.shape[0]
             step = key.step or 1
-            if start >= 0 and stop <= self.shape[0]:
-                return Sound(np.ndarray.__getitem__(self,(key,channel)),self.samplerate)
+            if start>=0 and stop<=self.shape[0]:
+                return Sound(np.ndarray.__getitem__(self, (key, channel)),
+                             self.samplerate)
             else:
-                bpad = Sound(zeros((-start,self.shape[1])))
-                apad = Sound(zeros((stop-self.shape[0],self.shape[1])))
-                if step==-1:
-                    return Sound(vstack((apad,flipud(asarray(self)),bpad)),self.samplerate)
-                return Sound(vstack((bpad,asarray(self),apad)),self.samplerate)
+                startpad = max(-start, 0)
+                endpad = max(stop-self.shape[0], 0)
+                startmid = max(start, 0)
+                endmid = min(stop, self.shape[0])
+                atstart = zeros((startpad, self.shape[1]))
+                atend = zeros((endpad, self.shape[1]))
+                return Sound(vstack((atstart,
+                                     asarray(self)[startmid:endmid:step],
+                                     atend)), self.samplerate)
         if not slicedims.all():
-            raise DimensionMismatchError('Slicing',*[units.get_unit(d) for d in sliceattr])
+            raise DimensionMismatchError('Slicing',
+                                        *[units.get_unit(d) for d in sliceattr])
         
         start = key.start or 0*msecond
         stop = key.stop or self.duration
@@ -341,15 +358,59 @@ class Sound(BaseSound, numpy.ndarray):
             padding = zeros((L - len(self), self.nchannels))
             return Sound(concatenate((self, padding)), samplerate=self.samplerate)
 
-    def shifted(self, duration):
+    def shifted(self, duration, fractional=False, filter_length=2048):
         '''
         Returns the sound delayed by duration, which can be the number of
-        samples or a length of time in seconds.
+        samples or a length of time in seconds. Normally, only integer
+        numbers of samples will be used, but if ``fractional=True`` then
+        the filtering method from
+        `http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html <http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html>`__
+        will be used (introducing some small numerical errors). With this
+        method, you can specify the ``filter_length``, larger values are
+        slower but more accurate, especially at higher frequencies. The large
+        default value of 2048 samples provides good accuracy for sounds with
+        frequencies above 20 Hz, but not for lower frequency sounds. If you are
+        restricted to high frequency sounds, a smaller value will be more
+        efficient. Note that if ``fractional=True`` then
+        ``duration`` is assumed to be a time not a number of samples.
         '''
-        if not isinstance(duration, int):
-            duration = int(rint(duration*self.samplerate))
-        y = vstack((zeros((duration, self.nchannels)), self))
-        return Sound(y, samplerate=self.samplerate)
+        if not fractional:
+            if not isinstance(duration, int):
+                duration = int(rint(duration*self.samplerate))
+            if duration>=0:
+                y = vstack((zeros((duration, self.nchannels)), self))
+                return Sound(y, samplerate=self.samplerate)
+            else:
+                return self[-duration:, :]
+        else:
+            if self.nchannels>1:
+                sounds = [self.channel(i).shifted(duration, fractional=True, filter_length=filter_length) for i in xrange(self.nchannels)]
+                return Sound(array(sounds), samplerate=self.samplerate)
+            # Adapted from
+            # http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html
+            delay = duration*self.samplerate
+            if delay>=0:
+                idelay = int(delay)
+            elif delay<0:
+                idelay = -int(-delay)
+            delay -= idelay
+            centre_tap = filter_length // 2
+            t = arange(filter_length)
+            x = t-delay
+            if abs(round(delay)-delay)<1e-10:
+                tap_weight = array(x==centre_tap, dtype=float)
+            else:
+                sinc = sin(pi*(x-centre_tap))/(pi*(x-centre_tap))
+                window = 0.54-0.46*cos(2.0*pi*(x+0.5)/filter_length) # Hamming window
+                tap_weight = window*sinc
+            if filter_length<256:
+                y = convolve(tap_weight, self.flatten())
+            else:
+                y = fftconvolve(tap_weight, self.flatten())
+            y = y[filter_length/2:-filter_length/2]
+            sound = Sound(y, self.samplerate)
+            sound = sound.shifted(idelay)
+            return sound
 
     def repeat(self, n):
         '''
@@ -395,7 +456,8 @@ class Sound(BaseSound, numpy.ndarray):
         x = array((2 ** 15 - 1) * clip(self / a, -1, 1), dtype=int16)
         if self.nchannels==1:
             x.shape = x.size
-        x = pygame.sndarray.make_sound(x)
+        # Make sure pygame receives an array in C-order        
+        x = pygame.sndarray.make_sound(ascontiguousarray(x))
         x.play()
         if sleep:
             time.sleep(self.duration)
@@ -562,6 +624,31 @@ class Sound(BaseSound, numpy.ndarray):
         newsound = self.copy()
         newsound.level = level
         return newsound
+    
+    def get_maxlevel(self):
+        return amax(self.level)*dB
+    
+    def set_maxlevel(self, level):
+        self.level += level-self.maxlevel
+        
+    maxlevel = property(fget=get_maxlevel, fset=set_maxlevel, doc='''
+        Can be used to set or get the maximum level of a sound. For mono
+        sounds, this is the same as the level, but for multichannel sounds
+        it is the maximum level across the channels. Relative level differences
+        will be preserved. The specified level should be a value in dB, and it
+        is assumed that the unit of the sound is Pascals. 
+        ''')
+
+    def atmaxlevel(self, level):
+        '''
+        Returns the sound with the maximum level across channels set to the
+        given level. Relative level differences will be preserved. The specified
+        level should be a value in dB and it is assumed that the unit of the
+        sound is Pascals.
+        '''
+        newsound = self.copy()
+        newsound.maxlevel = level
+        return newsound
             
     def ramp(self, when='onset', duration=10*ms, envelope=None, inplace=True):
         '''
@@ -593,7 +680,7 @@ class Sound(BaseSound, numpy.ndarray):
         if when == 'onset' or when == 'both':
             target[:sz, :] *= multiplier
         if when == 'offset' or when == 'both':
-            target[-sz:, :] *= multiplier[::-1]
+            target[target.nsamples-sz:, :] *= multiplier[::-1]
         return target
     
     def ramped(self, when='onset', duration=10*ms, envelope=None):
@@ -637,7 +724,9 @@ class Sound(BaseSound, numpy.ndarray):
             frequency.shape = (nchannels, 1)
         if phase.size==nchannels:
             phase.shape =(nchannels, 1)
-        x = sin(phase+2.0*pi*frequency*tile(arange(0, duration, 1)/samplerate,(nchannels,1))).T
+        t = arange(0, duration, 1)/samplerate
+        t.shape = (t.size, 1) # ensures C-order (in contrast to tile(...).T )
+        x = sin(phase + 2.0 * pi * frequency * tile(t, (1, nchannels)))
         return Sound(x, samplerate)
 
     @staticmethod
@@ -762,6 +851,49 @@ class Sound(BaseSound, numpy.ndarray):
         Returns brown noise, i.e :func:`powerlawnoise` with alpha=2
         '''
         return Sound.powerlawnoise(duration,2.0,samplerate=samplerate,normalise=False)
+    
+    @staticmethod
+    def irns(delay, gain, niter, duration, samplerate=None, nchannels=1):
+        '''
+        Returns an IRN_S noise. The iterated ripple noise is obtained trough
+        a cascade of gain and delay filtering. 
+        For more details: see Yost 1996 or chapter 15 in Hartman Sound Signal Sensation.
+        '''
+        samplerate = get_samplerate(samplerate)
+        noise=Sound.whitenoise(duration)
+        splrate=noise.samplerate
+        x=array(noise.T)[0]
+        IRNfft=np.fft.fft(x)
+        Nspl,spl_dur=len(IRNfft),float(1.0/splrate)
+        w=2*pi*fftfreq(Nspl,spl_dur)
+        d=float(delay)
+        for k in range(1,niter+1):
+            nchoosek=factorial(niter)/(factorial(niter-k)*factorial(k))
+            IRNfft+=nchoosek*(gain**k)*IRNfft*exp(-1j*w*k*d)
+        IRNadd = np.fft.ifft(IRNfft)
+        x=real(IRNadd)
+        return Sound(x,samplerate)
+    
+    @staticmethod
+    def irno(delay, gain, niter, duration, samplerate=None, nchannels=1):
+        '''
+        Returns an IRN_O noise. The iterated ripple noise is obtained many attenuated and
+        delayed version of the original broadband noise. 
+        For more details: see Yost 1996 or chapter 15 in Hartman Sound Signal Sensation.
+        '''
+        samplerate = get_samplerate(samplerate)
+        noise=Sound.whitenoise(duration)
+        splrate=noise.samplerate
+        x=array(noise.T)[0]
+        IRNadd=np.fft.fft(x)
+        Nspl,spl_dur=len(IRNadd),float(1.0/splrate)
+        w=2*pi*fftfreq(Nspl,spl_dur)
+        d=float(delay)
+        for k in range(1,niter+1):
+            IRNadd+=(gain**k)*IRNadd*exp(-1j*w*k*d)
+        IRNadd = np.fft.ifft(IRNadd)
+        x=real(IRNadd)
+        return Sound(x, samplerate)
 
     @staticmethod
     def click(duration, peak=None, samplerate=None, nchannels=1):
@@ -801,6 +933,72 @@ class Sound(BaseSound, numpy.ndarray):
         x=numpy.zeros((duration,nchannels))
         return Sound(x, samplerate)
 
+    @staticmethod
+    def vowel(vowel=None, formants=None, pitch=100*Hz, duration=1*second,
+          samplerate=None, nchannels=1):
+        '''
+        Returns an artifically created spoken vowel sound (following the 
+        source-filter model of speech production) with a given ``pitch``.
+        
+        The vowel can be specified by either providing ``vowel`` as a string
+        ('a', 'i' or 'u') or by setting ``formants`` to a sequence of formant
+        frequencies.
+        
+        The returned sound is normalized to a maximum amplitude of 1.
+        
+        The implementation is based on the MakeVowel function written by Richard
+        O. Duda, part of the Auditory Toolbox for Matlab by Malcolm Slaney:
+        http://cobweb.ecn.purdue.edu/~malcolm/interval/1998-010/                
+        '''    
+        
+        samplerate = get_samplerate(samplerate)
+        duration = get_duration(duration, samplerate)
+        
+        if not (vowel or formants):
+            raise ValueError('Need either a vowel or a list of formants')
+        elif (vowel and formants):
+            raise ValueError('Cannot use both vowel and formants')
+            
+        if vowel:
+            if vowel == 'a' or vowel == '/a/':
+                formants = (730.0*Hz, 1090.0*Hz, 2440.0*Hz)
+            elif vowel == 'i' or vowel == '/i/':
+                formants = (270.0*Hz, 2290.0*Hz, 3010.0*Hz)
+            elif vowel == 'u' or vowel == '/u/':
+                formants = (300.0*Hz, 870.0*Hz, 2240.0*Hz)
+            else:
+                raise ValueError('Unknown vowel: "%s"' % (vowel))            
+        
+        points = np.arange(0, duration - 1, samplerate / pitch)
+            
+        indices = np.floor(points).astype(int)
+        
+        y = np.zeros(duration)
+    
+        y[indices] = (indices + 1) - points
+        y[indices + 1] = points - indices
+        
+        # model the sound source (periodic glottal excitation)  
+        a = np.exp(-250.*Hz * 2 * np.pi / samplerate)
+        y = lfilter([1],[1, 0, -a * a], y.copy())
+        
+        # model the filtering by the vocal tract
+        bandwidth = 50.*Hz
+        
+        for f in formants:
+            cft = f / samplerate
+            q = f / bandwidth
+            rho = np.exp(-np.pi * cft / q)
+            theta = 2 * np.pi * cft * np.sqrt(1 - 1/(4.0 * q * q))
+            a2 = -2 * rho * np.cos(theta)
+            a3 = rho * rho
+            y = lfilter([1 + a2 + a3], [1, a2, a3], y.copy()) 
+        
+        #normalize sound
+        data = y / np.max(np.abs(y), axis=0)        
+        data.shape = (data.size, 1)
+        return Sound(np.tile(data, (nchannels, 1)),  samplerate=samplerate)
+
     @staticmethod
     def sequence(*args, **kwds):
         '''
@@ -831,7 +1029,7 @@ class Sound(BaseSound, numpy.ndarray):
         normalised to 1. The samplewidth keyword can be 1 or 2 to save the data as
         8 or 16 bit samples.
         '''
-        ext=filename.split('.')[-1]
+        ext = filename.split('.')[-1].lower()
         if ext=='wav':
             import wave as sndmodule
         elif ext=='aiff' or ext=='aifc':
@@ -876,29 +1074,28 @@ class Sound(BaseSound, numpy.ndarray):
         Load the file given by filename and returns a Sound object. 
         Sound file can be either a .wav or a .aif file.
         '''
-        ext=filename.split('.')[-1]
+        ext = filename.split('.')[-1].lower()
         if ext=='wav':
             import wave as sndmodule
         elif ext=='aif' or ext=='aiff':
             import aifc as sndmodule
         else:
             raise NotImplementedError('Can only load aif or wav soundfiles')
-        def everyOther (v, offset=0, channels=2):
-            return [v[i] for i in range(offset, len(v), channels)]
-        wav = sndmodule.open (filename, "r")
-        (nchannels, sampwidth, framerate, nframes, comptype, compname) = wav.getparams ()
-        frames = wav.readframes (nframes * nchannels)
+        wav = sndmodule.open(filename, "r")
+        nchannels, sampwidth, framerate, nframes, comptype, compname = wav.getparams()
+        frames = wav.readframes(nframes * nchannels)
         typecode = {2:'h', 1:'B'}[sampwidth]
-        out = struct.unpack_from ("%d%s" % (nframes * nchannels,typecode), frames)
+        out = frombuffer(frames, dtype=dtype(typecode))
         scale = {2:2 ** 15, 1:2 ** 7-1}[sampwidth]
         meanval = {2:0, 1:2**7}[sampwidth]
         
-        data=zeros((nframes,nchannels))
+        data = zeros((nframes, nchannels))
         for i in range(nchannels):
-            data[:,i]=array(everyOther(out,offset=i,channels=nchannels))
-            data[:,i]/=scale
-            data[:,i]-=meanval
-        return Sound(data,samplerate=framerate*Hz)
+            data[:, i] = out[i::nchannels]
+            data[:, i] /= scale
+            data[:, i] -= meanval
+        
+        return Sound(data, samplerate=framerate*Hz)
 
     def __reduce__(self):
         return (_load_Sound_from_pickle, (asarray(self), float(self.samplerate)))
@@ -941,10 +1138,13 @@ whitenoise = Sound.whitenoise
 powerlawnoise = Sound.powerlawnoise
 pinknoise = Sound.pinknoise
 brownnoise = Sound.brownnoise
+irns = Sound.irns
+irno = Sound.irno
 tone = Sound.tone
 harmoniccomplex = Sound.harmoniccomplex
 click = Sound.click
 clicks = Sound.clicks
 silence = Sound.silence
 sequence = Sound.sequence
+vowel = Sound.vowel
 loadsound = Sound.load
