@@ -1,6 +1,7 @@
-from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
-                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
-                    reshape, sum, log
+#from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
+#                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
+#                    reshape, sum, log
+from brian import *
 from brian.tools.statistics import firing_rate, get_gamma_factor
 try:
     from playdoh import *
